#include <math.h>
#include "ttc.h"

double line_seg_point_dist_3d (double x1,double y1,double z1,double x2,double y2,double z2,
                               double x,double y,double z)
{
double bot,dist,t,xn,yn,zn;
  if (x1==x2&&y1==y2&&z1==z2)
    {
    xn = x1;
    yn = y1;
    zn = z1;
    }
  else
    {
    bot=(x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2);
    t=((x1-x)*(x1-x2)+(y1-y)*(y1-y2)+(z1-z)*(z1-z2))/bot;
    t=MAX(t,0.0);
    t=MIN(t,1.0);
    xn=x1+t*(x2-x1);
    yn=y1+t*(y2-y1);
    zn=z1+t*(z2-z1);
  }
  dist=sqrt((xn-x)*(xn-x)+(yn-y)*(yn-y)+(zn-z)*(zn-z));
  return dist;
}

void line_seg_point_near_3d(double x1,double y1,double z1,double x2,double y2,double z2,
        double x,double y,double z,double *xn,double *yn,double *zn,double *dist,double *t)
{
  double bot;
  if(x1==x2&&y1==y2&&z1==z2)
    {
    *t = 0.0;
    *xn = x1;
    *yn = y1;
    *zn = z1;
    }
  else
    {
    bot=(x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2);
    *t = (+(x1-x)*(x1-x2)+(y1-y)*(y1-y2)+(z1-z)*(z1-z2))/bot;
    if(*t<0.0)
      {
      *t = 0.0;
      *xn = x1;
      *yn = y1;
      *zn = z1;
      }
    else if(*t>1.0)
      {
      *t = 1.0;
      *xn = x2;
      *yn = y2;
      *zn = z2;
      }
    else
      {
      *xn = x1 + *t * ( x2 - x1 );
      *yn = y1 + *t * ( y2 - y1 );
      *zn = z1 + *t * ( z2 - z1 );
      }
  }
  *dist=sqrt((*xn-x)*(*xn-x)+(*yn-y)*(*yn-y)+(*zn-z)*(*zn-z));
  return;
}

//Determines colinearity of 3 points in 3D
//Input:double x1,y1,z1,x2,y2,z2,x3,y3,z3 (the 3 points)
//Output:double: colinearity (range is 0 to 1)
//Note:Colinearity is defined as the value from dividing the area of a triangle formed by
//the 3 points by the area of an equilateral triangle formed from the 2 most seperate points.
//a return of 0 indicates total linearity and 1 indicates maximum non-linearity.
double points_colin_3d ( double x1, double y1, double z1, double x2, double y2,
  double z2, double x3, double y3, double z3 )
{
  double area,area2,colin,perim,side;
  area=triangle_area_3d(x1,y1,z1,x2,y2,z2,x3,y3,z3);
  area=fabs(area);
  if (area==0.0)
     {
     colin=0.0;
     }
  else
     {
     perim=enorm0_3d(x1,y1,z1,x2,y2,z2)+enorm0_3d(x2,y2,z2,x3,y3,z3)
     +enorm0_3d(x3,y3,z3,x1,y1,z1);
     side=perim/3.0;
     area2=0.25*sqrt(3.0)*side*side;
     colin=area/area2;
     }
return colin;
}

//Determines the area of a triangle in 3d
//Input:double x1,y1,z1,x2,y2,z2,x3,y3,z3 (the vertices of the triangle)
//Output:double (the area of the triangle
double triangle_area_3d ( double x1, double y1, double z1, double x2, 
 double y2, double z2, double x3, double y3, double z3 )
{
double a,alpha,area,b,base,c,dot,height;
dot=(x2-x1)*(x3-x1)+(y2-y1)*(y3-y1)+(z2-z1)*(z3-z1);
base=enorm0_3d(x1,y1,z1,x2,y2,z2);
if(base==0.0)
    {
    height=0.0;
    }
else
    {
    alpha=dot/(base*base);
    a=x3-x1-alpha*(x2-x1);
    b=y3-y1-alpha*(y2-y1);
    c=z3-z1-alpha*(z2-z1);
    height=sqrt(a*a+b*b+c*c);
    }
area=0.5*base*height;
return area;
}

double enorm0_3d(double x0,double y0,double z0,double x1,double y1,double z1)
{
  double value;
  value = sqrt((x1-x0)*(x1-x0)+(y1-y0)*(y1-y0)+(z1-z0)*(z1-z0));
  return value;
}

void cross_3d ( float x1, float y1, float z1, float x2, float y2, float z2,
  float *x3, float *y3, float *z3 )
{
  *x3 = y1 * z2 - z1 * y2;
  *y3 = z1 * x2 - x1 * z2;
  *z3 = x1 * y2 - y1 * x2;

  return;
}

//VECTOR_UNIT_3D normalizes a vector in 3D.
void vector_unit_3d(double *x1,double *y1,double *z1)
{
  float temp;

  temp = sqrt ( (*x1) * (*x1) + (*y1) * (*y1) + (*z1) * (*z1) );

  if ( temp != 0.0 ) {
    *x1 = *x1 / temp;
    *y1 = *y1 / temp;
    *z1 = *z1 / temp;
  }
  return;
}
